@@ -22,6 +22,7 @@ import graphlab.aggregate
 import graphlab.canvas
 import graphlab.toolkits
 import graphlab.toolkits.clustering as clustering
+distances = graphlab.util._distances_utility()
 
 from graphlab.toolkits import evaluation
 import graphlab.toolkits.vowpal_wabbit as vowpal_wabbit
@@ -62,6 +63,7 @@ from graphlab.toolkits.image_analysis import image_analysis
 from graphlab.data_structures.sgraph import load_sgraph, load_graph
 from graphlab.data_structures.sframe import load_sframe, get_spark_integration_jar_path
 from graphlab.toolkits._model import load_model
+from graphlab.data_structures.DBConnection import connect_odbc, get_libodbc_path, set_libodbc_path
 
 
 
@@ -72,14 +74,15 @@ from graphlab.connect.main import stop as _stop
 import graphlab.connect.main as glconnect
 
 # python egg version
-__VERSION__ = '1.0.1'
-version = '1.0.1'
+__VERSION__ = '1.1'
+version = '1.1'
 
 from graphlab.util import get_newest_version
 from graphlab.util import perform_version_check
 from graphlab.util import get_environment_config
 from graphlab.util import get_runtime_config
 from graphlab.util import set_runtime_config
+from graphlab.util import get_graphlab_object_type
 
 from graphlab.version_info import version
 from graphlab.version_info import __VERSION__
