@@ -5,7 +5,7 @@ import array
 
 class TaskView(graphlab.canvas.views.base.BaseView):
 
-    def __init__(self, obj, params):
+    def __init__(self, obj, params=None):
         super(TaskView, self).__init__(obj, params)
         self.cached_sketch = None
 
