@@ -28,8 +28,9 @@ from graphlab.connect import _get_metric_tracker
 CONFIG_SECTION = 'AWS'
 DEFAULT_CIDR_RULE = '0.0.0.0/0'  # Open to the entire internet.
 DEFAULT_INSTANCE_TYPE = 'm3.xlarge'
-JSON_BLOB_PATH_FORMAT = "/gl_api/api/v1/cloud/aws/instances/%s?client_version=%s&region=%s"
+JSON_BLOB_PATH_FORMAT = "/gl_api/api/v1/cloud/aws/instances/%s?client_version=%s&region=%s&product_key=%s"
 GRAPHLAB_NAME = 'GraphLab'
+VALID_REGIONS = [r.name for r in boto.ec2.get_regions('ec2')]
 
 # Port 9000 & 9001 - used when running in server mode: only server running with external client.
 # Port 9002 & 9003 - used for metrics (deprecated).
@@ -44,6 +45,8 @@ logging.getLogger('boto').setLevel(logging.CRITICAL)
 
 __LOGGER__ = logging.getLogger(__name__)
 
+class LicenseValidationException(Exception):
+    pass
 
 class _Ec2Instance:
     def __init__(self, public_dns_name, instance_id, instance, region):
@@ -105,7 +108,6 @@ class _Ec2GraphLabServer(glserver.RemoteServer):
         return self.auth_token
 
     def stop(self):
-        _get_metric_tracker().flush()
         _stop_instances([self.instance_id], self.region)
 
 
@@ -161,8 +163,8 @@ def launch_EC2(instance_type=DEFAULT_INSTANCE_TYPE, region=None, CIDR_rule=None,
     ----------
     instance_type : string, optional
         The EC2 instance type to launch. We support `all instance types
-        <http://aws.amazon.com/ec2/instance-types/#Instance_Types>`_ except:
-        't2.micro', 't2.small', 't2.medium' and 'm3.medium'.
+        <http://aws.amazon.com/ec2/instance-types/#Instance_Types>`_ that are not
+        micros, smalls or mediums.
 
     region : string, optional
         The `AWS region
@@ -220,25 +222,31 @@ def launch_EC2(instance_type=DEFAULT_INSTANCE_TYPE, region=None, CIDR_rule=None,
         (client_public_key, client_secret_key) = get_public_secret_key_pair()
         __LOGGER__.debug('Launching server with public key: %s' % server_public_key)
 
-    server = _Ec2GraphLabServer(instance_type, region, CIDR_rule, auth_token, security_group, tags,
+    try: 
+        server = _Ec2GraphLabServer(instance_type, region, CIDR_rule, auth_token, security_group, tags,
                                 server_public_key, server_secret_key)
 
-    try:
-        # Once the EC2 instance starts up, the client still need to wait for graphlab engine to start up.
-        # So allow for a larger than normal number of ping failures. This is especially important when launching
-        # in other AWS regions (since longer latency and longer download from S3 for engine binary)
-        server.start(num_tolerable_ping_failures=120)
-    except Exception as e:
-        __LOGGER__.error("Unable to successfully connect to GraphLab Server on EC2 instance: '%s'."
+        try:
+            # Once the EC2 instance starts up, the client still need to wait for graphlab engine to start up.
+            # So allow for a larger than normal number of ping failures. This is especially important when launching
+            # in other AWS regions (since longer latency and longer download from S3 for engine binary)
+            server.start(num_tolerable_ping_failures=120)
+    
+        except Exception as e:
+            __LOGGER__.error("Unable to successfully connect to GraphLab Server on EC2 instance: '%s'."
                          " Please check AWS Console to make sure any EC2 instances launched have"
                          " been terminated." % e)
-        server.stop()
-        raise e
-
+            server.stop()
+            raise e
+    
+    except LicenseValidationException as e:
+        # catch exception and print license check hint message here instead of raise
+        __LOGGER__.info(e)
+        return 
     # Create the client
     num_tolerable_ping_failures = 3
     client = Client([], server.get_server_addr(), num_tolerable_ping_failures, public_key=client_public_key,
-                    secret_key=client_secret_key, server_public_key=server_public_key)
+                secret_key=client_secret_key, server_public_key=server_public_key)
     auth_token = server.get_auth_token()
     if auth_token:
         client.add_auth_method_token(auth_token)
@@ -246,6 +254,7 @@ def launch_EC2(instance_type=DEFAULT_INSTANCE_TYPE, region=None, CIDR_rule=None,
     glconnect._assign_server_and_client(server, client)
 
 
+
 def set_credentials(access_key_id, secret_access_key):
     """
     Sets the AWS credential environment variables.
@@ -390,19 +399,23 @@ def _ec2_factory(instance_type, region=None, CIDR_rule=None, security_group_name
     nessary), determine arguments to start up the EC2 instance (i.e. AMI id and user data),
     actually start up the EC2 instance, wait for it, and applies AWS tags.
     '''
-
     # Get and validate the product key
     product_key = graphlab.product_key.get_product_key()
     if (not graphlab.product_key.is_product_key_valid(product_key)):
-        __LOGGER__.error("Product Key validation failed, please confirm your product key is correct. "
-                         "If you believe this key to be valid, please contact support@graphlab.com")
-        return
-
+        raise LicenseValidationException("Product Key validation failed, please confirm your product key is correct.\
+ If you believe this key to be valid, please contact support@graphlab.com")
+    
     # Set default values for parameters.
     if(region is None):
         region = _get_region_from_config()
         if(region is None):
             region = 'us-west-2'
+        else:
+            __LOGGER__.info('Read region from config file.')
+
+    if (region not in VALID_REGIONS):
+        raise Exception("%s is not a valid region." % region)
+
     # If a CIDR rule is specified, a security group must also be specified.
     if CIDR_rule is not None and security_group is None:
         __LOGGER__.error('If you specify a CIDR rule, you must also specify an existing security group.')
@@ -426,18 +439,18 @@ def _ec2_factory(instance_type, region=None, CIDR_rule=None, security_group_name
         # This is the first AWS contact, so the most likely cause is env vars aren't set.
         __LOGGER__.debug('EC2 Response Error: %s' % e)
         raise Exception('EC2 response error. Please verify your AWS credentials. To configure your '
-                        'AWS credentials use graphlab.aws.set_credentials or set the '
-                        'AWS_ACCESS_KEY_ID and AWS_SECRET_ACCESS_KEY environment variables'
-                        'Exception: %s' % e)
+                'AWS credentials use graphlab.aws.set_credentials or set the '
+                'AWS_ACCESS_KEY_ID and AWS_SECRET_ACCESS_KEY environment variables'
+                'Exception: %s' % e)
     except boto.exception.NoAuthHandlerFound as e:
-            raise Exception('AWS configuration not found. Please configure your AWS credentials '
-                            'using graphlab.aws.set_credentials or by setting AWS_ACCESS_KEY_ID '
-                            'and AWS_SECRET_ACCESS_KEY environment variables')
+        raise Exception('AWS configuration not found. Please configure your AWS credentials '
+                'using graphlab.aws.set_credentials or by setting AWS_ACCESS_KEY_ID '
+                'and AWS_SECRET_ACCESS_KEY environment variables')
 
     # If security group doesn't exist, create it.
     if security_group is None:
         security_group = conn.create_security_group(security_group_name,
-                                                    'Only open ports needed for GraphLab Server')
+                'Only open ports needed for GraphLab Server')
 
     # Configure the security group
     try:
@@ -449,8 +462,8 @@ def _ec2_factory(instance_type, region=None, CIDR_rule=None, security_group_name
         if not "<Code>InvalidPermission.Duplicate</Code>" in str(e):
             raise e
 
-    # Determine AMI id, as well as engine and os URL
-    if (config.mode != 'PROD' and 'GRAPHLAB_TEST_AMI_ID' in os.environ and 'GRAPHLAB_TEST_ENGINE_URL' in os.environ):
+    # Determine AMI id, as well as engine, os URL, and hash value
+    if (config.mode != 'PROD' and 'GRAPHLAB_TEST_AMI_ID' in os.environ and 'GRAPHLAB_TEST_ENGINE_URL' in os.environ and 'GRAPHLAB_TEST_HASH_KEY' in os.environ):
         # unit-test mode, don't involve webservice to retrieve AMI, instead use environment variables
         ami_id = os.environ['GRAPHLAB_TEST_AMI_ID']
         engine_url = os.environ['GRAPHLAB_TEST_ENGINE_URL']
@@ -459,9 +472,10 @@ def _ec2_factory(instance_type, region=None, CIDR_rule=None, security_group_name
         json_blob['ami_id'] = ami_id
         json_blob['engine_url'] = engine_url
         json_blob['os_url'] = ''
+        json_blob['hash_key'] = os.environ['GRAPHLAB_TEST_HASH_KEY']
     else:
         # Get the info to start a EC2 from the GraphLab Server
-        json_blob_path = JSON_BLOB_PATH_FORMAT % (instance_type, config.version, region)
+        json_blob_path = JSON_BLOB_PATH_FORMAT % (instance_type, config.version, region,product_key)
         for (param_name, param_value) in ami_service_parameters.items():
             json_blob_path += "&%s=%s" % (str(param_name), str(param_value))
         json_blob_url = config.graphlab_server + json_blob_path
@@ -472,29 +486,34 @@ def _ec2_factory(instance_type, region=None, CIDR_rule=None, security_group_name
             json_blob = json.loads(graphlab_server_response.read())
         except:
             raise Exception('Unable to successfully retrieve correct EC2 image to launch for this '
-                            'version. This could be a temporary problem. Please try again in a few '
-                            'minutes. If the problem persists please contact support@graphlab.com')
-    __LOGGER__.debug("web service return: %s" % json_blob)
+                    'version. This could be a temporary problem. Please try again in a few '
+                    'minutes. If the problem persists please contact support@graphlab.com')
+        __LOGGER__.debug("web service return: %s" % json_blob)
+        
+        if json_blob.get('error'):
+            raise LicenseValidationException(json_blob.get('error'))
 
     if 'ami_id' not in json_blob or json_blob['ami_id'] is None:
         raise Exception("Unable to successfully retrieve correct EC2 image to launch. Please try "
-                        "again later or contact support@graphlab.com. Error received:'%s'"
-                        % json_blob['message'])
+                "again later or contact support@graphlab.com. Error received:'%s'"
+                % json_blob.get('message'))
     ami_id = json_blob['ami_id']
 
-    # Add json_blob to user_data and set the product key
+    # Add json_blob to user_data and set the product key and hash key
     user_data.update(json_blob)
     user_data['product_key'] = product_key
 
+    user_data['hash_key'] = json_blob.get('hash_key', 'NO_HASH_VALUE')
+
     # Check for testing override of os_url param.
     if (config.mode != 'PROD' and 'GRAPHLAB_TEST_OS_URL' in os.environ):
         user_data['os_url'] = os.environ['GRAPHLAB_TEST_OS_URL']
 
     run_instances_args =  {
-        'security_group_ids' : [ security_group_name ],
-        'user_data' : json.dumps(user_data),
-        'instance_type' : instance_type
-    }
+            'security_group_ids' : [ security_group_name ],
+            'user_data' : json.dumps(user_data),
+            'instance_type' : instance_type
+            }
 
     if num_hosts != 1:
         run_instances_args['min_count'] = num_hosts
@@ -517,8 +536,8 @@ def _ec2_factory(instance_type, region=None, CIDR_rule=None, security_group_name
         # Report
         for i in instances:
             __LOGGER__.info("Launching an %s instance in the %s availability zone, with id: %s."
-                            " You will be responsible for the cost of this instance."
-                            % (i.instance_type, i.placement, i.id))
+                    " You will be responsible for the cost of this instance."
+                    % (i.instance_type, i.placement, i.id))
 
         # Wait for all host(s) to say they're done starting up.
         while True:
@@ -542,7 +561,7 @@ def _ec2_factory(instance_type, region=None, CIDR_rule=None, security_group_name
         if instances:
             _stop_instances([i.id for i in instances] , self.region)
         raise Exception("Unable to launch EC2 instance: '%s'. Please check AWS Console to make "
-                         " sure any EC2 instances launched have been terminated." % e)
+                " sure any EC2 instances launched have been terminated." % e)
 
     # Add tags to this instance(s).
     if(tags is None):
