@@ -1,4 +1,12 @@
 """
+
+NOTE: DO NOT USE THIS FOR ANY NEW CODE. THIS IS ONLY HERE FOR 
+BACKWARDS COMPATIBILITY REASONS FOR GRAPHLAB DATA PIPELINES
+
+Instead, use graphlab_util.cloudpickle if you want the current
+version of cloudpickle
+
+
 This class is defined to override standard pickle functionality
 
 The goals of it follow:
@@ -115,8 +123,6 @@ printMemoization = False
 
 useForcedImports = True #Should I use forced imports for tracking?
 
-
-
 class CloudPickler(pickle.Pickler):
 
     dispatch = pickle.Pickler.dispatch.copy()
@@ -764,11 +770,32 @@ class CloudPickler(pickle.Pickler):
             return
         self.dispatch[email.LazyImporter] = self.__class__.save_unsupported
 
+    def inject_unity_proxy(self):
+        gl = sys.modules.get('graphlab')
+        if not gl:
+            return
+        ## Make sure the unity objects are not picklable ##
+        self.dispatch[gl.SArray] = self.__class__.save_unsupported
+        self.dispatch[gl.SFrame] = self.__class__.save_unsupported
+        self.dispatch[gl.SGraph] = self.__class__.save_unsupported
+        self.dispatch[gl.Graph] = self.__class__.save_unsupported
+        self.dispatch[gl.Sketch] = self.__class__.save_unsupported
+        self.dispatch[gl.Model] = self.__class__.save_unsupported
+
+        ## Make sure the underlying cython objects are not picklable ##
+        self.dispatch[gl.cython.cy_sarray.UnitySArrayProxy] = self.__class__.save_unsupported
+        self.dispatch[gl.cython.cy_sframe.UnitySFrameProxy] = self.__class__.save_unsupported
+        self.dispatch[gl.cython.cy_sketch.UnitySketchProxy] = self.__class__.save_unsupported
+        self.dispatch[gl.cython.cy_graph.UnityGraphProxy] = self.__class__.save_unsupported
+        self.dispatch[gl.cython.cy_model.UnityModel] = self.__class__.save_unsupported
+        self.dispatch[gl.cython.cy_ipc.PyCommClient] = self.__class__.save_unsupported
+
     def inject_addons(self):
         """Plug in system. Register additional pickling functions if modules already loaded"""
         self.inject_numpy()
         self.inject_timeseries()
         self.inject_email()
+        self.inject_unity_proxy()
 
     """Python Imaging Library"""
     def save_image(self, obj):
