@@ -20,7 +20,7 @@ def print_callback(val):
         # I have to intrude rather deep into IPython to make it behave
         if have_ipython:
             if InteractiveShell.initialized():
-                IPython.display.publish_display_data('graphlab.callback', {'text/plain':val,'text/html':val + '<br/>'})
+                IPython.display.publish_display_data('graphlab.callback', {'text/plain':val,'text/html':'<pre>' + val + '</pre>'})
                 success = True
     except:
         pass
