@@ -149,7 +149,7 @@ class SArray(object):
                     elif len(data.shape) > 2:
                         raise TypeError("Cannot convert Numpy arrays of greater than 2 dimensions")
 
-                elif (isinstance(data, str)):
+                elif (isinstance(data, str) or isinstance(data, unicode)):
                     # if it is a file, we default to string
                     dtype = str
 
@@ -159,7 +159,7 @@ class SArray(object):
             elif (HAS_NUMPY and isinstance(data, numpy.ndarray)) or isinstance(data, list) or isinstance(data, array.array):
                 with cython_context():
                     self.__proxy__.load_from_iterable(data, dtype, ignore_cast_failure)
-            elif (isinstance(data, str)):
+            elif (isinstance(data, str) or isinstance(data, unicode)):
                 internal_url = make_internal_url(data)
                 with cython_context():
                     self.__proxy__.load_autodetect(internal_url, dtype)
@@ -565,17 +565,40 @@ class SArray(object):
         the SArray. If the key is a slice, this returns an SArray with the
         sliced rows. See the GraphLab Create User Guide for usage examples.
         """
-        if type(other) is SArray:
-            if len(other) != len(self):
+        sa_len = len(self)
+        
+        if type(other) is int:
+            if other < 0:
+                other += sa_len
+            if other >= sa_len:
+                raise IndexError("SFrame index out of range")
+
+            try:
+                lb, ub, value_list = self._getitem_cache
+                if lb <= other < ub:
+                    return value_list[other - lb]
+                
+            except AttributeError:
+                pass
+            
+            # Not in cache, need to grab it
+            block_size = 1024 * (32 if self.dtype() in [int, long, float] else 4)
+
+            block_num = int(other // block_size)
+
+            lb = block_num * block_size
+            ub = min(sa_len, lb + block_size)
+            
+            val_list = list(SArray(_proxy = self.__proxy__.copy_range(lb, 1, ub)))
+            self._getitem_cache = (lb, ub, val_list)
+            return val_list[other - lb]
+        
+        elif type(other) is SArray:
+            if len(other) != sa_len:
                 raise IndexError("Cannot perform logical indexing on arrays of different length.")
             with cython_context():
                 return SArray(_proxy = self.__proxy__.logical_filter(other.__proxy__))
-        elif type(other) is int:
-            if other < 0:
-                other = len(self) + other
-            if other >= len(self):
-                raise IndexError("SFrame index out of range")
-            return list(SArray(_proxy = self.__proxy__.copy_range(other, 1, other+1)))[0]
+                    
         elif type(other) is slice:
             start = other.start
             stop = other.stop
@@ -583,14 +606,15 @@ class SArray(object):
             if start is None:
                 start = 0
             if stop is None:
-                stop = len(self)
+                stop = sa_len
             if step is None:
                 step = 1
             # handle negative indices
             if start < 0:
-                start = len(self) + start
+                start = sa_len + start
             if stop < 0:
-                stop = len(self) + stop
+                stop = sa_len + stop
+
             return SArray(_proxy = self.__proxy__.copy_range(start, step, stop))
         else:
             raise IndexError("Invalid type to use for indexing")
@@ -1035,6 +1059,9 @@ class SArray(object):
         fn : function
             The function to transform each element. Must return exactly one
             value which can be cast into the type specified by ``dtype``.
+            This can also be a toolkit extension function which is compiled
+            as a native shared library using SDK.
+
 
         dtype : {None, int, float, str, list, array.array, dict, graphlab.Image}, optional
             The data type of the new SArray. If ``None``, the first 100 elements
@@ -1063,6 +1090,32 @@ class SArray(object):
         dtype: int
         Rows: 3
         [2, 4, 6]
+
+        Using native toolkit extension function:
+
+        .. code-block:: c++
+
+            #include <graphlab/sdk/toolkit_function_macros.hpp>
+            #include <cmath>
+
+            using namespace graphlab;
+            double logx(const flexible_type& x, double base) {
+              return log((double)(x)) / log(base);
+            }
+
+            BEGIN_FUNCTION_REGISTRATION
+            REGISTER_FUNCTION(logx, "x", "base");
+            END_FUNCTION_REGISTRATION
+
+        compiled into example.so
+
+        >>> import example
+
+        >>> sa = graphlab.SArray([1,2,4])
+        >>> sa.apply(lambda x: example.logx(x, 2))
+        dtype: float
+        Rows: 3
+        [0.0, 1.0, 2.0]
         """
         if (type(fn) == str):
             fn = "LUA" + fn
@@ -1402,7 +1455,6 @@ class SArray(object):
         with cython_context():
             if self.dtype() == graphlab.Image:
                 import graphlab.extensions as extensions
-                extensions.list_functions()
                 return extensions.generate_mean(self)
             else:
                 return self.__proxy__.mean()
@@ -1580,6 +1632,21 @@ class SArray(object):
         -------
         out : SArray[graphlab.Image]
             The SArray converted to the type 'graphlab.Image'.
+
+        See Also
+        --------
+        astype, str_to_datetime, datetime_to_str
+
+        Examples
+        --------
+        The MNIST data is scaled from 0 to 1, but our image type only loads integer  pixel values
+        from 0 to 255. If we just convert without scaling, all values below one would be cast to
+        0.
+
+        >>> mnist_array = graphlab.SArray('http://s3.amazonaws.com/GraphLab-Datasets/mnist/mnist_vec_sarray')
+        >>> scaled_mnist_array = mnist_array * 255
+        >>> mnist_img_sarray = gl.SArray.pixel_array_to_image(scaled_mnist_array, 28, 28, 1, allow_rounding = True)
+
         """
         if(self.dtype() != array.array):
             raise TypeError("array_to_img expects SArray of arrays as input SArray")
@@ -1601,7 +1668,6 @@ class SArray(object):
         _mt._get_metric_tracker().track('sarray.pixel_array_to_img')
 
         import graphlab.extensions as extensions
-        extensions.list_functions()
         return extensions.vector_sarray_to_image_sarray(self, width, height, channels, undefined_on_failure)
 
     def _head_str(self, num_rows):
@@ -1609,7 +1675,6 @@ class SArray(object):
         Takes the head of SArray casted to string.
         """
         import graphlab.extensions as extensions
-        extensions.list_functions()
         return extensions._head_str(self, num_rows)
 
     def astype(self, dtype, undefined_on_failure=False):
@@ -2112,7 +2177,7 @@ class SArray(object):
             [datetime(2011, 1, 21, 7, 7, 21, tzinfo=GMT(0)),
              datetime(2010, 2, 5, 7, 8, 21, tzinfo=GMT(4.5)])
 
-         >>> sa.expand(column_name_prefix=None,limit=['day','year'])
+         >>> sa.split_datetime(column_name_prefix=None,limit=['day','year'])
             Columns:
                 day   int
                 year  int
