@@ -162,8 +162,8 @@ class SGraph(object):
     and edge lists stored in :class:`~graphlab.SFrames`. Columns of these
     SFrames not used as vertex IDs are assumed to be vertex or edge attributes.
 
-    Please see the `User Guide 
-    <http://www.graphlab.com/learn/userguide/index.html#Working_with_data_Graph_data>`_ 
+    Please see the `User Guide
+    <http://www.graphlab.com/learn/userguide/index.html#Working_with_data_Graph_data>`_
     for a more detailed introduction to creating and working with SGraphs.
 
     Parameters
@@ -540,10 +540,7 @@ class SGraph(object):
             dst_ids = [None] * len(src_ids)
 
         with cython_context():
-            if (len(src_ids) + len(dst_ids) + len(fields)) == 0:
-                sf = SFrame(_proxy=self.__proxy__.get_edges_lazy())
-            else:
-                sf = SFrame(_proxy=self.__proxy__.get_edges(src_ids, dst_ids, fields))
+            sf = SFrame(_proxy=self.__proxy__.get_edges(src_ids, dst_ids, fields))
 
         if (format == 'sframe'):
             return sf
@@ -844,6 +841,8 @@ class SGraph(object):
             (source_data, edge_data, target_data) and return a tuple of
             (new_source_data, new_edge_data, new_target_data). All variables in
             the both tuples must be of dict type.
+            This can also be a toolkit extension function which is compiled
+            as a native shared library using SDK.
 
         mutated_fields : list[str] | str
             Fields that ``triple_apply_fn`` will mutate. Note: columns that are
@@ -886,7 +885,44 @@ class SGraph(object):
         Apply the function to the SGraph.
 
         >>> g = g.triple_apply(degree_count_fn, mutated_fields=['degree'])
+
+
+        Using native toolkit extension function:
+
+        .. code-block:: c++
+
+            #include <graphlab/sdk/toolkit_function_macros.hpp>
+            #include <vector>
+
+            using namespace graphlab;
+            std::vector<variant_type> connected_components_parameterized(
+              std::map<std::string, flexible_type>& src,
+              std::map<std::string, flexible_type>& edge,
+              std::map<std::string, flexible_type>& dst,
+              std::string column) {
+                if (src[column] < dst[column]) dst[column] = src[column];
+                else src[column] = dst[column];
+                return {to_variant(src), to_variant(edge), to_variant(dst)};
+            }
+
+            BEGIN_FUNCTION_REGISTRATION
+            REGISTER_FUNCTION(connected_components_parameterized, "src", "edge", "dst", "column");
+            END_FUNCTION_REGISTRATION
+
+        compiled into example.so
+
+        >>> from example import connected_components_parameterized as cc
+        >>> e = gl.SFrame({'__src_id':[1,2,3,4,5], '__dst_id':[3,1,2,5,4]})
+        >>> g = gl.SGraph().add_edges(e)
+        >>> g.vertices['cid'] = g.vertices['__id']
+        >>> for i in range(2):
+        ...     g = g.triple_apply(lambda src, edge, dst: cc(src, edge, dst, 'cid'), ['cid'], ['cid'])
+        >>> g.vertices['cid']
+        dtype: int
+        Rows: 5
+        [4, 1, 1, 1, 4]
         '''
+
         assert inspect.isfunction(triple_apply_fn), "Input must be a function"
         if not (type(mutated_fields) is list or type(mutated_fields) is str):
             raise TypeError('mutated_fields must be str or list of str')
@@ -982,8 +1018,8 @@ class SGraph(object):
              ewidth=1, v_offset=0.03, h_offset=0., arrows=False,
              vertex_positions=None):
         """
-        show(vlabel=None, vlabel_hover=False, vcolor=[0.522, 0.741, 0.], highlight={}, highlight_color=[0.69, 0., 0.498], node_size=300, elabel=None, elabel_hover=False, ecolor=[0.37, 0.33, 0.33], ewidth=1, v_offset=0.03, h_offset=0., arrows=False, vertex_positions=None)                
-        
+        show(vlabel=None, vlabel_hover=False, vcolor=[0.522, 0.741, 0.], highlight={}, highlight_color=[0.69, 0., 0.498], node_size=300, elabel=None, elabel_hover=False, ecolor=[0.37, 0.33, 0.33], ewidth=1, v_offset=0.03, h_offset=0., arrows=False, vertex_positions=None)
+
         Visualize the SGraph with GraphLab :mod:`~graphlab.canvas`. This
         function starts Canvas if it is not already running. If the graph has
         already been plotted, this function will update the plot. SGraphs must
@@ -999,7 +1035,7 @@ class SGraph(object):
 
         vlabel_hover : bool, optional
             If True, vertex labels, if specified, appear only on mouse hover.
-            Otherwise (the default), vertex labels, if specified are always 
+            Otherwise (the default), vertex labels, if specified are always
             visible.
 
         vcolor : list[float], optional
@@ -1008,16 +1044,16 @@ class SGraph(object):
 
         highlight : dict or list or SArray, optional
             As a dict, mapping of Vertex ID to RGB color triple (list of float,
-            as in vcolor). 
+            as in vcolor).
 
             As a list or SArray (DEPRECATED): Vertex IDs to highlight in
             a different color.
 
         highlight_color : list[float], optional
             RGB triple for the color of highlighted vertices, when the
-            highlighted parameter is a list or SArray. Default is fuchsia 
+            highlighted parameter is a list or SArray. Default is fuchsia
             ([0.69, 0.,0.498]). For fine-grained control over vertex coloring,
-            use the `highlight` parameter with a dictionary of Vertex IDs and 
+            use the `highlight` parameter with a dictionary of Vertex IDs and
             color values.
 
         node_size : int, optional
@@ -1186,13 +1222,13 @@ class SGraph(object):
 #/*                            Module Function                             */
 #/*                                                                        */
 #/**************************************************************************/
-def load_graph(filename, format='binary'):
+def load_graph(filename, format='binary', delimiter='auto'):
     import warnings
     warnings.warn("load_graph has been renamed to load_sgraph. This function will be removed in the next release.", PendingDeprecationWarning)
     return load_sgraph(filename, format=format)
 
 
-def load_sgraph(filename, format='binary'):
+def load_sgraph(filename, format='binary', delimiter='auto'):
     """
     Load SGraph from text file or previously saved SGraph binary.
 
@@ -1205,11 +1241,16 @@ def load_sgraph(filename, format='binary'):
         Format to of the file to load.
 
         - 'binary': native graph format obtained from `SGraph.save`.
-        - 'snap': tab separated edge list format with comments, used in
+        - 'snap': tab or space separated edge list format with comments, used in
           the `Stanford Network Analysis Platform <http://snap.stanford.edu/snap/>`_.
         - 'csv': comma-separated edge list without header or comments.
         - 'tsv': tab-separated edge list without header or comments.
 
+    delimiter : str, optional
+        Specifying the Delimiter used in 'snap', 'csv' or 'tsv' format. Those
+        format has default delimiter, but sometimes it is useful to
+        overwrite the default delimiter.
+
     Returns
     -------
     out : SGraph
@@ -1237,14 +1278,20 @@ def load_sgraph(filename, format='binary'):
             proxy = glconnect.get_unity().load_graph(make_internal_url(filename))
             g = SGraph(_proxy=proxy)
         elif format is 'snap':
-            sf = SFrame.read_csv(filename, comment_char='#', delimiter='\t',
+            if delimiter == 'auto':
+                delimiter = '\t'
+            sf = SFrame.read_csv(filename, comment_char='#', delimiter=delimiter,
                                  header=False, column_type_hints=int)
             g = SGraph().add_edges(sf, 'X1', 'X2')
         elif format is 'csv':
-            sf = SFrame.read_csv(filename, header=False)
+            if delimiter == 'auto':
+                delimiter = ','
+            sf = SFrame.read_csv(filename, header=False, delimiter=delimiter)
             g = SGraph().add_edges(sf, 'X1', 'X2')
         elif format is 'tsv':
-            sf = SFrame.read_csv(filename, header=False, delimiter='\t')
+            if delimiter == 'auto':
+                delimiter = '\t'
+            sf = SFrame.read_csv(filename, header=False, delimiter=delimiter)
             g = SGraph().add_edges(sf, 'X1', 'X2')
         g.summary()  # materialize
         return g
