@@ -724,3 +724,16 @@ class Sketch(object):
                 return ret[keys[0]]
             else:
                 return ret
+
+    def cancel(self):
+      """
+      Cancels a background sketch computation immediately if one is ongoing.
+      Does nothing otherwise.
+
+      Examples
+      --------
+      >>> s = sa.sketch_summary(array, background=True)
+      >>> s.cancel()
+      """
+      with cython_context():
+        self.__proxy__.cancel()
