@@ -22,8 +22,8 @@ class Task(object):
     A Task encapsulates a block of code.
 
     A Task is a modular, composable, reusable component that encapsulates a
-    unit-of-work. It allows loose coupling between other Tasks, and is the
-    robust building block for building GraphLab Data Pipelines.
+    unit-of-work. It is the robust building block for building GraphLab Data
+    Pipelines.
 
     Tasks allow for code to logically reference inputs, outputs, and parameters
     by name, so the code is reusable in many places.
@@ -141,7 +141,7 @@ class Task(object):
     def clone(self, name):
         """
         Make a copy of the current Task with a new name. The new task will have
-        different name, but will share all other task metadata like inputs,
+        a different name, but will share all other task metadata like inputs,
         outputs, description, and code.
 
         Parameters
@@ -659,11 +659,13 @@ class Task(object):
 
     def set_params(self, names, delete=False):
         """
-        Add/set parameter(s) for this Task.
+        Add/set simple parameter(s) for this Task.
+
+        Parameters can only be strings or numbers, not Python objects.
 
         Parameters
         ----------
-        params : dict
+        names : dict
             Dictionary with key/values (strings), merged into current params.
 
         delete : bool, optional
@@ -672,9 +674,30 @@ class Task(object):
 
         Returns
         -------
-        self : Task       
+        self : Task
+
+        Examples
+        --------
+        To set params using a list of names:
+
+            >>> task.set_params(['one', 'two', 'three'])
+
+        To set params using a dict:
+            
+            >>> task.set_params({'x':1, 'y':2})
+
+        To delete/clear all params:
+
+            >>> task.set_params(['one', 'two', 'three'])
+            >>> task.set_params(None, delete=True)
+
+        To delete a specific param:
+            
+            >>> task.set_params(['one', 'two', 'three'])
+            >>> task.set_params(['one'], delete=True) # param named 'one' deleted
+
         """
-        if names is None:
+        if names is None and delete is False:
             raise TypeError('names is a required parameter for set_params and cannot be None.')
 
         if isinstance(names, list):
@@ -682,6 +705,9 @@ class Task(object):
             self._set_params(params=dictify_params, delete=delete)
         elif isinstance(names, dict):
             self._set_params(params=names, delete=delete)
+        elif names is None and delete is True:
+            # delete all params
+            self._set_params(None, True)
         else:
             raise TypeError('Unknown type for names parameter. names must be list or dict.')
         return self
