@@ -286,7 +286,7 @@ class Ec2ExecutionEnvironment(ExecutionEnvironment):
         # Make sure we can write to S3
         try:
             conn = _connect_s3(**credentials)
-            bucket = conn.get_bucket(job.environment.s3_bucket)
+            bucket = conn.get_bucket(job.environment.s3_bucket, validate=False)
             k = _s3_key(bucket)
             k.key = s3_state_key
             k.set_contents_from_string('Pending')
