@@ -281,7 +281,8 @@ class Job(object):
         --------
         canvas
         """
-        _canvas.get_target().state.set_selected_variable(('Jobs', self.name))
+        filename = self._session._get_filename_from_name(self.name)
+        _canvas.get_target().state.set_selected_variable(('Jobs', filename))
         return _canvas.show()
 
     @staticmethod
@@ -317,11 +318,9 @@ class Job(object):
         return output.getvalue()
 
     def __getstate__(self):
-        # session attribute gets added when loaded from disk
-        # make sure to remove it when pickling
         odict = dict.copy(self.__dict__)
-        if hasattr(odict, '_session'):
-            del odict._session
+        if '_session' in odict:
+            del odict['_session']
         return odict
 
     def __str__(self):
@@ -500,7 +499,7 @@ class Ec2Job(Job):
             credentials['aws_access_key_id'] = self.environment.aws_access_key
             credentials['aws_secret_access_key'] = self.environment.aws_secret_key
         s3_conn = _connect_s3(**credentials)
-        bucket = s3_conn.get_bucket(self.s3_bucket)
+        bucket = s3_conn.get_bucket(self.s3_bucket, validate=False)
         k = bucket.get_key(self.s3_state_key)
 
         # The key only gets deleted on success.
@@ -547,7 +546,7 @@ class Ec2Job(Job):
             credentials['aws_access_key_id'] = self.environment.aws_access_key
             credentials['aws_secret_access_key'] = self.environment.aws_secret_key
         s3_conn = _connect_s3(**credentials)
-        bucket = s3_conn.get_bucket(self.s3_bucket)
+        bucket = s3_conn.get_bucket(self.s3_bucket, validate=False)
         k = bucket.get_key(self.s3_status_key)
 
         try:
@@ -607,7 +606,7 @@ class Ec2Job(Job):
             credentials['aws_access_key_id'] = self.environment.aws_access_key
             credentials['aws_secret_access_key'] = self.environment.aws_secret_key
         s3_conn = _connect_s3(**credentials)
-        bucket = s3_conn.get_bucket(self.s3_bucket)
+        bucket = s3_conn.get_bucket(self.s3_bucket, validate=False)
         k = bucket.get_key(self.s3_log_key)
 
         if not k:
