@@ -7,10 +7,11 @@ import os
 from requests import get, post, ConnectionError
 from subprocess import Popen
 from time import sleep
-from datetime import datetime
+from datetime import datetime, timedelta
 import urllib
 from subprocess import Popen, PIPE, STDOUT
 import boto.ec2.elb
+import boto.ec2.cloudwatch
 from boto.exception import BotoServerError
 import boto.iam.connection
 
@@ -29,6 +30,10 @@ from _predictive_service import MAX_CREATE_TIMEOUT_SECS
 _logger = _logging.getLogger(__name__)
 
 ENV_TYPE = 'environment type'
+# The maximum number of datapoints CloudWatch will return for a given query
+CW_MAX_DATA_POINTS = 1440
+# The minimum allowable period (seconds) for CloudWatch
+CW_MIN_PERIOD = 60
 
 # disable spurious output from requests library
 _logging.getLogger("requests").setLevel(_logging.WARNING)
@@ -119,6 +124,9 @@ class PredictiveServiceEnvironment:
         except Exception as e:
             raise RuntimeError("Cannot get metrics from service, error: %s" % resp.text)
 
+    def _get_metrics(self, name, start_time, end_time, period):
+        pass
+
     def terminate(self, del_flag):
         pass
 
@@ -271,6 +279,20 @@ class Ec2PredictiveServiceEnvironment(PredictiveServiceEnvironment):
         self.certificate_is_self_signed = certificate_is_self_signed
         self.aws_credentials = aws_credentials
 
+    def __repr__(self):
+        ret = ""
+        ret += 'Ec2PredictiveServiceEnvironment:\n'
+        ret += '\tload_balancer_dns_name: %s\n' % str(self.load_balancer_dns_name)
+        ret += '\tregion: %s\n' % str(self.region)
+        ret += '\tlog_path: %s\n' % str(self.log_path)
+        ret += '\tcertificate_name: %s\n' % str(self.certificate_name)
+        if self.certificate_name:
+            ret += '\tcertificate_is_self_signed: %s\n' % str(self.certificate_is_self_signed)
+        return ret
+
+    def __str__(self):
+        return self.__repr__()
+
     @staticmethod
     def launch(name, ec2_config, s3_predictive_object_path, admin_key, ssl_credentials,
                aws_credentials, started):
@@ -297,15 +319,18 @@ class Ec2PredictiveServiceEnvironment(PredictiveServiceEnvironment):
         lb = None
         try:
             # Determine host ids and availability zones used.
-            zones, host_ids  = set(), []
+            zones, host_ids, is_vpc = set(), [], False
             if ec2_config.num_hosts == 1:
                 host_ids = [ec2_hosts.instance_id]
                 zones.add(ec2_hosts.instance.placement)
+                is_vpc = bool(ec2_hosts.instance.vpc_id)
             else:
                 for i in ec2_hosts:
                     zones.add(i.instance.placement)
                     host_ids.append(i.instance_id)
 
+                is_vpc = bool(ec2_hosts[0].instance.vpc_id)
+
             certificate_name, certificate_is_self_signed = None, None
             if ssl_credentials:
                 # Using HTTPS
@@ -339,7 +364,7 @@ class Ec2PredictiveServiceEnvironment(PredictiveServiceEnvironment):
 
 
             # If we're running in a VPC (i.e. not "EC2-Classic") we need to attach a security group.
-            if ec2_hosts[0].instance.vpc_id:
+            if is_vpc:
                 # Look up security group id
                 security_group = None
                 while ((datetime.now() - started).total_seconds() < MAX_CREATE_TIMEOUT_SECS):
@@ -405,6 +430,49 @@ class Ec2PredictiveServiceEnvironment(PredictiveServiceEnvironment):
                              'state': host.state, 'reason': host.reason_code } )
         return result
 
+    def _get_metrics(self, name, start_time, end_time, period=300):
+        # Calculate time range for period determination/sanity-check
+        delta = end_time - start_time
+        delta_seconds = ( delta.days * 24 * 60 * 60 ) + delta.seconds + 1 #round microseconds up
+
+        # Determine min period as the smallest multiple of 60 that won't result in too many data points
+        min_period = 60 * int(delta_seconds / CW_MAX_DATA_POINTS / 60)
+        if ((delta_seconds / CW_MAX_DATA_POINTS) % 60) > 0:
+            min_period += 60
+       
+        # set the min period if necessary
+        if period < min_period:
+            period = min_period
+        if period < CW_MIN_PERIOD:
+            period = CW_MIN_PERIOD
+        
+        # get cloud watch handle
+        cw = self._get_cloudwatch_handle()
+
+        # get AWS/ELB requests, latency, num of healthy hosts, and num of cache keys
+        num_requests = cw.get_metric_statistics(period, start_time, end_time,
+                        'RequestCount', 'AWS/ELB', ['Sum'], {"LoadBalancerName": name}, 'Count')
+
+        latency = cw.get_metric_statistics(period, start_time, end_time,
+                        'Latency', 'AWS/ELB', ['Average'], {"LoadBalancerName": name}, 'Seconds')
+
+        num_healthy_hosts = cw.get_metric_statistics(period, start_time, end_time,
+                        'HealthyHostCount', 'AWS/ELB', ['Average'], {"LoadBalancerName": name}, 'Count')
+
+        num_keys_in_cache = cw.get_metric_statistics(period, start_time, end_time,
+                        'num_keys_in_cache', name, ['Maximum'], {}, 'Count')
+
+        return {'num_requests' : num_requests, 'latency' : latency, 'num_healthy_hosts' : num_healthy_hosts,
+                'num_keys_in_cache' : num_keys_in_cache}
+
+
+    def _get_cloudwatch_handle(self):
+        try:
+            conn = boto.ec2.cloudwatch.connect_to_region(self.region, **self.aws_credentials)
+            return conn
+        except Exception as e:
+            _logger.error("Unable to connect to to CloudWatch in region '%s'" % self.region)
+        raise Exception("Cannot connect to CloudWatch in region '%s'." % self.region)
 
     def _get_load_balancer_handle(self):
         try:
@@ -505,5 +573,21 @@ class TestStubEnv(PredictiveServiceEnvironment):
     def _get_state(self):
         return {ENV_TYPE: 'TestStubEnv'}
 
+    def _get_metrics(self, name, start_time, end_time, period):
+        num_requests = []
+        latency = []
+        num_healthy_hosts = []
+        num_keys_in_cache = []
+        time_delta = timedelta(seconds=period)
+        start = start_time
+        while start < end_time:
+            num_requests.append({'Sum': 1.0, 'Timestamp': start, 'Unit':'Count'}) 
+            latency.append({'Average': 1.0, 'Timestamp': start, 'Unit':'Seconds'})
+            num_healthy_hosts.append({'Average': 1.0, 'Timestamp': start, 'Unit':'Count'})
+            num_keys_in_cache.append({'Sum': 1.0, 'Timestamp': start, 'Unit':'Count'}) 
+            start += time_delta
+        return {'num_requests': num_requests, 'latency': latency, 'num_healthy_hosts': num_healthy_hosts,
+                        'num_keys_in_cache': num_keys_in_cache}
+
     def poke(self):
         pass
