@@ -1,6 +1,8 @@
 import graphlab.connect as _mt
 from _job import Job as _Job
 from graphlab.deploy import Task as _Task, _Pipeline, _default_session
+from graphlab.connect.aws._ec2 import LicenseValidationException
+
 import environment as _environment
 import _executionenvironment as _env
 import graphlab as _gl
@@ -20,7 +22,7 @@ __LOGGER__ = _logging.getLogger(__name__)
 
 
 def create(tasks=None, name=None, environment=None, function=None, function_arguments=None, required_packages=None):
-    """ 
+    """
     Creates a Job and begins executing it. The Job can be defined by either
     specifying a list of tasks, with optional bindings, or with a function,
     with arguments defined. Each Job also needs to know where to run, and that
@@ -37,8 +39,8 @@ def create(tasks=None, name=None, environment=None, function=None, function_argu
     name : str
         Name for this execution (names the returned Job). Default is environment name + timestamp.
 
-    environment : :class:`~graphlab.deploy.environment.EC2` | 
-                  :class:`~graphlab.deploy.environment.Hadoop` | 
+    environment : :class:`~graphlab.deploy.environment.EC2` |
+                  :class:`~graphlab.deploy.environment.Hadoop` |
                   :class:`~graphlab.deploy.environment.LocalAsync`
         Optional environment for execution. This would commonly hold access
         keys, launch locations etc.  Also included in the environment object is
@@ -82,7 +84,7 @@ def create(tasks=None, name=None, environment=None, function=None, function_argu
       inspecting the Job object returned.
 
     Examples
-    -------- 
+    --------
     Creating a Job using a function instead of Tasks is easy. Just define a
     function and then use it when calling job.create.
 
@@ -92,50 +94,37 @@ def create(tasks=None, name=None, environment=None, function=None, function_argu
         >>>
         >>> job = graphlab.deploy.job.create(
         >>>             function=sum_four,
-        >>>             function_arguments=[{'one':1, 'two':2, 
+        >>>             function_arguments=[{'one':1, 'two':2,
         >>>                 'three':3, 'four':4}])
         >>>
         >>> results = job.get_results() # SFrame with aggregated results
 
-    Using an SFrame to specify arguments:
+    Using an SFrame to specify multiple sets of arguments:
         >>> def mult_three(one, two, three):
         >>>     return {'product': one * two * three}
         >>>
-        >>> sf = graphlab.SFrame(data={'one':[1], 'two':[2], 'three':[3]})
-        >>> job = graphlab.deploy.job.create(function=mult_three, 
+        >>> sf = graphlab.SFrame(data={'one':[1,5], 'two':[2,6], 'three':[3,7]})
+        >>> job = graphlab.deploy.job.create(function=mult_three,
         >>>                                  function_arguments=sf)
         >>>
-        >>> results = job.get_results() # SFrame with aggregated results
-        >>> print results
-        >>> Columns:
-        >>>         index   int
-        >>>         input   dict
-        >>>         result  dict
-        >>>         status  str
-        >>>         start_time      datetime
-        >>>         end_time        datetime
-        >>>         message str
-        >>> 
-        >>> Rows: 1
-        >>> 
-        >>> Data:
-        >>> +-------+--------------------------------+----------------+-----------+
-        >>> | index |             input              |     result     |   status  |
-        >>> +-------+--------------------------------+----------------+-----------+
-        >>> |   0   | {'three': 3, 'two': 2, 'on ... | {'product': 6} | Completed |
-        >>> +-------+--------------------------------+----------------+-----------+
-        >>> +---------------+---------------+---------+
-        >>> |   start_time  |    end_time   | message |
-        >>> +---------------+---------------+---------+
-        >>> | 1412389681.61 | 1412389681.61 |         |
-        >>> +---------------+---------------+---------+
-        >>> [1 rows x 7 columns]
-        >>>
+        >>> +----+--------------------------------+------------------+-----------+
+        >>> | id |             input              |      result      |   status  |
+        >>> +----+--------------------------------+------------------+-----------+
+        >>> | 0  | {'three': 3, 'two': 2, 'on ... |  {'product': 6}  | Completed |
+        >>> | 1  | {'three': 7, 'two': 6, 'on ... | {'product': 210} | Completed |
+        >>> +----+--------------------------------+------------------+-----------+
+        >>> +---------------------------+---------------------------+---------+
+        >>> |         start_time        |          end_time         | message |
+        >>> +---------------------------+---------------------------+---------+
+        >>> | 2014-11-17 11:06:38+00:00 | 2014-11-17 11:06:38+00:00 |         |
+        >>> | 2014-11-17 11:06:40+00:00 | 2014-11-17 11:06:40+00:00 |         |
+        >>> +---------------------------+---------------------------+---------+
+        >>> [2 rows x 7 columns]
 
     Each entry in the tasks list could be a pair with a dictionary of bindings
     for that entry. For example:
 
-        >>> tasks = [('task1', {'input':'s3://big-file'}), 
+        >>> tasks = [('task1', {'input':'s3://big-file'}),
         >>>          ('task2', {'input':'/localfile'})]
         >>> graphlab.deploy.job.create(tasks, name='with-bindings')
 
@@ -160,7 +149,7 @@ def create(tasks=None, name=None, environment=None, function=None, function_argu
 
         elif not isinstance(environment, _environment.Environment):
             raise Exception("Unknown type of environment")
-        
+
         if environment is None:
             raise TypeError("Environment cannot be loaded correctly with name '%s', please confirm this environment exists by calling graphlab.deploy.environments." % environment)
 
@@ -217,13 +206,24 @@ def create(tasks=None, name=None, environment=None, function=None, function_argu
         combiner.set_params({'num' : len(bindings)})
         tasks.append(combiner)
         _gl.deploy.tasks.delete(combiner)
-        tracker.track('graphlab.deploy.job.create.fn', value=1)
+        tracker.track('deploy.job.create.fn', value=1)
 
     # now make the artifacts a list of objects
     if not isinstance(tasks, list):
         # not a list, let's turn it into a list
         tasks = [tasks]
 
+    # if Environment object missing num_hosts attribute, set to 1
+    if not hasattr(environment, 'num_hosts'):
+        environment.num_hosts = 1
+
+    # If environment.num_hosts > 1 and not using model_parameter_search or parallel_for_each then
+    # reset num_host = 1, since multiple hosts will not be used.
+    if environment.num_hosts != 1 and \
+            all(map(lambda x : isinstance(x, _Task), tasks)):
+                __LOGGER__.warn("Ignoring Environment.num_hosts value since execution will occur only on one host. Using num_hosts=1 for this execution.")
+                environment.num_hosts = 1
+    
     # add required packages to first task in execution
     # ensures the packages will be present on execution
     if required_packages is not None:
@@ -240,7 +240,7 @@ def create(tasks=None, name=None, environment=None, function=None, function_argu
             names = task.name
         name = 'job-%s-%s-%s' % (names, environment.name, _time.time())
 
-    # if using fn= parameter, we need to wait until name has been determined to 
+    # if using fn= parameter, we need to wait until name has been determined to
     # set the results_path, so now that the name is settled, set results_path
     if combiner is not None:
         results_path = _get_results_path(environment, name)
@@ -300,7 +300,7 @@ def create(tasks=None, name=None, environment=None, function=None, function_argu
             if binding is not None:
                 _apply_binding_to_task(clone, binding)
 
-            # if environment is not local then write out any outputs not bound to a location to an 
+            # if environment is not local then write out any outputs not bound to a location to an
             # intermediate location, so any subsequent steps can find the output
             _validate_output_to_environment(clone, environment, validation_msgs)
 
@@ -310,51 +310,54 @@ def create(tasks=None, name=None, environment=None, function=None, function_argu
 
     num_tasks = len(cloned_artifacts)
     if isinstance(environment, _environment.Local):
-        tracker.track('graphlab.deploy.job.create.local', value=1, properties={'num_tasks':num_tasks})
+        tracker.track('deploy.job.create.local', value=1, properties={'num_tasks':num_tasks})
         env = _env.LocalExecutionEnvironment()
     elif isinstance(environment, _environment.LocalAsync):
-        tracker.track('graphlab.deploy.job.create.localasync', value=1, properties={'num_tasks':num_tasks})
+        tracker.track('deploy.job.create.localasync', value=1, properties={'num_tasks':num_tasks})
         env = _env.LocalAsynchronousEnvironment()
     elif isinstance(environment, _environment.EC2):
-        tracker.track('graphlab.deploy.job.create.ec2', value=1, properties={'num_tasks':num_tasks})
+        tracker.track('deploy.job.create.ec2', value=1, properties={'num_tasks':num_tasks})
         # name the ec2 instance the job name
         if not environment.tags:
             environment.tags = {}
         if not 'Name' in environment.tags:
             environment.tags['Name'] = name
         environment.tags['Job'] = name
-        environment.num_hosts = 1 # make sure environment.num_hosts, needed for EC2 launch
         env = _env.Ec2ExecutionEnvironment()
     elif isinstance(environment, _environment.Hadoop):
-        tracker.track('graphlab.deploy.job.create.hadoop', value=1, properties={'num_tasks':num_tasks})
+        tracker.track('deploy.job.create.hadoop', value=1, properties={'num_tasks':num_tasks})
         env = _env.HadoopExecutionEnvironment()
     else:
         raise Exception("Validation Failed: Unknown execution environment.")
-    
+
     if len(validation_msgs) > 0:
         for msg in validation_msgs:
             __LOGGER__.error(msg)
         raise RuntimeError("Validation Failed: output(s) not set to appropriate location for execution environment. See logs for more details.")
+    
+    try:
+        __LOGGER__.info("Validation complete. Job: '%s' ready for execution" % name)
+        job = env.run(_session, cloned_artifacts, name, environment)
+        _session.register(job)
+        job.save() # save the job once prior to returning.
 
-    __LOGGER__.info("Validation complete. Job: '%s' ready for execution" % name)
-    job = env.run(_session, cloned_artifacts, name, environment)
-    _session.register(job)
-    job.save() # save the job once prior to returning.
+        # add a .get_results() method to this job.
+        if function is not None:
+            job.get_results = _types.MethodType(_get_results, job)
 
-    # add a .get_results() method to this job.
-    if function is not None:
-        job.get_results = _types.MethodType(_get_results, job)
+        return job
+    except LicenseValidationException as e:
+        # catch exception and print license check hint message here instead of raise
+        __LOGGER__.info(e)
 
-    return job
-
-def _get_results_path(environment, job_name):
+def _get_results_path(environment, job_name, folder_name = 'stored_results'):
     """
     Decide on a place to store the results of the job based on the environment passed in.
     """
     dirname = job_name
-    results_path = _os.path.join(_tempfile.gettempdir(), dirname, 'stored_results')
+    results_path = _os.path.join(_tempfile.gettempdir(), dirname, folder_name)
     if isinstance(environment, _environment.EC2):
-        
+
         # get credentials
         if not hasattr(environment, 'aws_access_key'):
             # no access key in environment, try to get from get_credentials
@@ -367,29 +370,29 @@ def _get_results_path(environment, job_name):
             access_key = environment.aws_access_key
             secret_key = environment.aws_secret_key
 
-        path = _os.path.join(environment.s3_bucket, environment.s3_log_folder_path, dirname, 'stored_results')
+        path = _os.path.join(environment.s3_bucket, environment.s3_log_folder_path, dirname, folder_name)
         results_path = "s3://%s:%s:%s" % (access_key, secret_key, path)
     elif isinstance(environment, _environment.Hadoop):
         # get the root path in hdfs land, to support actually launching in HDFS
         if environment.get_config_dir() is not None:
             root = _subprocess.check_output(
-                        ["hdfs", "--config", _os.path.expanduser(environment.get_config_dir()), 
+                        ["hdfs", "--config", _os.path.expanduser(environment.get_config_dir()),
                             "getconf", "-confKey", "fs.defaultFS"]).rstrip()
         else:
             root = _subprocess.check_output(["hdfs", "getconf", "-confKey", "fs.defaultFS"]).rstrip()
-        
+
         # get the username for running the Hadoop job.
         user = _pwd.getpwuid(_os.getuid()).pw_name
         if 'HADOOP_USER_NAME' in _os.environ:
             user = _os.environ['HADOOP_USER_NAME']
-        dirname = dirname + "-stored_results"
+        dirname = dirname + '-' + folder_name
         results_path = _os.path.join(root, 'user', user, dirname)
-    
+
     return results_path
 
 def _get_results(self):
     """
-    Retrieve the results of the job execution by retrieving the output 
+    Retrieve the results of the job execution by retrieving the output
     of the 'collector' Task.
     """
     status = self.get_status()
@@ -429,7 +432,7 @@ def _wrapper_task(task):
         status = 'Failed'
         end = _datetime.datetime.now()
         raise e
-    finally: 
+    finally:
         task.outputs['output'] = _gl.SFrame({'input':[params], 'result':[result], 'start_time':[start], 'end_time':[end], 'status':[status], 'message':[message]})
 
 def _combiner_task(task):
@@ -443,17 +446,18 @@ def _combiner_task(task):
     results_sa = gl.SArray(results_list)
     results = results_sa.unpack(column_name_prefix=None)
     results = results[['input', 'result', 'status', 'start_time', 'end_time', 'message']]
+    results = results.add_row_number()
     task.outputs['results'] = results
 
 def _apply_binding_to_task(task, binding, allow_object=False):
     """
-    Helper method to apply bindings to a given task. This function modifies the given task object and returns 
+    Helper method to apply bindings to a given task. This function modifies the given task object and returns
     it modified.
 
-    When specifying bindings, there is no qualification for a binding being an input, output, or param, so when 
-    trying to apply the binding we need to try each. A TypeError is thrown whenever the name for an input is already 
-    applied to another type of input (ex. set_input('foo') called, then later set_param('foo','bar') will throw a 
-    TypeError. So it is expected that the name for each binding throw a TypeError for the other types of slot names 
+    When specifying bindings, there is no qualification for a binding being an input, output, or param, so when
+    trying to apply the binding we need to try each. A TypeError is thrown whenever the name for an input is already
+    applied to another type of input (ex. set_input('foo') called, then later set_param('foo','bar') will throw a
+    TypeError. So it is expected that the name for each binding throw a TypeError for the other types of slot names
     (so if param named 'foo', then TypeError should be thrown for set_input('foo') and set_output('foo')
     """
     for (param_name, param_value) in binding.iteritems():
@@ -467,7 +471,7 @@ def _apply_binding_to_task(task, binding, allow_object=False):
                     if not hasattr(task, '_local_binding') or task._local_binding is None:
                         task._local_binding = {}
                     task._local_binding[param_name] = param_value
-                else:                    
+                else:
                     task.set_inputs({param_name : param_value})
                 bound = True
                 __LOGGER__.debug("Applied binding named: '%s' as input, with value: '%s'" % (param_name, param_value))
