@@ -11,7 +11,8 @@ from graphlab.toolkits._internal_utils import _SGraphFromJsonTree
 from graphlab.toolkits._internal_utils import _raise_error_evaluation_metric_is_valid
 from graphlab.toolkits._internal_utils import _raise_error_if_not_sframe
 from graphlab.toolkits._internal_utils import _raise_error_if_column_exists
-
+from graphlab.toolkits._internal_utils import _check_categorical_option_type
+from graphlab.toolkits._internal_utils import _map_unity_proxy_to_object
 
 
 DEFAULT_HYPER_PARAMETER_RANGE = {
@@ -19,8 +20,8 @@ DEFAULT_HYPER_PARAMETER_RANGE = {
     'step_size': 0.3,
     'min_loss_reduction': [0, 1, 10],
     'min_child_weight': 0.1,
-    'row_subsample': 1,
-    'column_subsample': 1,
+    'row_subsample': 1.0,
+    'column_subsample': 1.0,
     'max_iterations': [10, 50, 100]
 }
 
@@ -55,9 +56,11 @@ class BoostedTreesClassifier(_Classifier):
     sparse features (text data), or with categorical variables that encode a 
     large number of categories.
 
-    An instance of this model can be created using
-    :py:func:`~graphlab.boosted_trees_classifier.create`. This object cannot 
-    be constructed directly.
+    This model cannot be constructed directly.  Instead, use 
+    :func:`graphlab.boosted_trees_classifier.create` to create an instance of 
+    this model. 
+    Additional details on parameter options and code samples are available in 
+    the documentation for the create function.
 
     See Also
     --------
@@ -347,12 +350,78 @@ class BoostedTreesClassifier(_Classifier):
         >>> m.predict(testdata, output_type='margin')
         """
         _mt._get_metric_tracker().track('toolkit.classifier.boosted_trees_classifier.predict')
-        if output_type not in ['class', 'margin', 'probability']:
-            raise ValueError("Output type '{}' is not supported.".format(output_type) + \
-                      " Please select 'probability', 'class' or 'margin'.")
+
+        _check_categorical_option_type('output_type', output_type, ['class', 'margin', 'probability'])
         return super(_Classifier, self).predict(dataset,
                                                     output_type = output_type)
 
+    def predict_topk(self, dataset, output_type="probability", k=3):
+        """
+        Return top-k predictions for the ``dataset``, using the trained model.
+        Predictions are returned as an SFrame with three columns: `row_id`,
+        `class`, and `probability`, `margin`,  or `rank`, depending on the ``output_type``
+        parameter. Input dataset size must be the same as for training of the model.
+
+        Parameters
+        ----------
+        dataset : SFrame
+            A dataset that has the same columns that were used during training.
+            If the target column exists in ``dataset`` it will be ignored
+            while making predictions.
+
+        output_type : {'probability', 'rank', 'margin'}, optional
+            Choose the return type of the prediction:
+
+            - `probability`: Probability associated with each label in the prediction.
+            - `rank`       : Rank associated with each label in the prediction.
+            - `margin`     : Margin associated with each label in the prediction.
+
+        k : int, optional
+            Number of classes to return for each input example.
+
+        Returns
+        -------
+        out : SFrame
+            An SFrame with model predictions.
+
+        See Also
+        --------
+        predict, classify, evaluate
+
+        Examples
+        --------
+        >>> pred = m.predict_topk(validation_data, k=3)
+        >>> pred
+        +--------+-------+-------------------+
+        | row_id | class |   probability     |
+        +--------+-------+-------------------+
+        |   0    |   4   |   0.995623886585  |
+        |   0    |   9   |  0.0038311756216  |
+        |   0    |   7   | 0.000301006948575 |
+        |   1    |   1   |   0.928708016872  |
+        |   1    |   3   |  0.0440889261663  |
+        |   1    |   2   |  0.0176190119237  |
+        |   2    |   3   |   0.996967732906  |
+        |   2    |   2   |  0.00151345680933 |
+        |   2    |   7   | 0.000637513934635 |
+        |   3    |   1   |   0.998070061207  |
+        |  ...   |  ...  |        ...        |
+        +--------+-------+-------------------+
+        [35688 rows x 3 columns]
+        """
+        _mt._get_metric_tracker().track('toolkit.classifier.boosted_trees_classifier.predict_topk')
+        _raise_error_if_not_sframe(dataset, "dataset")
+        _check_categorical_option_type('output_type', output_type, ['rank', 'margin', 'probability'])
+        options = dict()
+        options.update({'model': self.__proxy__,
+                        'model_name': self.__name__,
+                        'dataset': dataset,
+                        'output_type': output_type,
+                        'topk': k,
+                        'missing_value_action': 'error'})
+        target = _graphlab.toolkits._main.run('supervised_learning_predict_topk', options)
+        return _map_unity_proxy_to_object(target['predicted'])
+
     def classify(self, dataset, missing_value_action='impute'):
         """
         Return a classification, for each example in the ``dataset``, using the
@@ -419,8 +488,8 @@ def create(dataset, target,
            verbose=True,
            **kwargs):
     """
-    Create a (binary) classifier model of type 
-    :class:`~graphlab.boosted_trees_classifier.BoostedTreesClassifier` using 
+    Create a (binary or multi-class) classifier model of type
+    :class:`~graphlab.boosted_trees_classifier.BoostedTreesClassifier` using
     gradient boosted trees (sometimes known as GBMs).
 
     Parameters
@@ -430,8 +499,11 @@ def create(dataset, target,
         The target column must be of numerical type (int, float).
 
     target : str
-        The name of prediction target column in input SFrame ``dataset``.
-        This column must have be of numeric type (int, float).
+        Name of the column containing the target variable. The values in this
+        column must of string or integer type.  String target variables are
+        autumatically mapped to integers in alphabetical order of the variable values.
+        For example, a target variable with 'cat', 'dog', and 'foosa' as possible
+        values is mapped to 0, 1, and, 2 respectively.
 
     features : list[str], optional
         A list of columns names of features used for training the model.
