@@ -31,7 +31,8 @@ def create(dataset, target, features=None, l2_penalty=0.01, l1_penalty=0.0,
     convergence_threshold = _DEFAULT_SOLVER_OPTIONS['convergence_threshold'],
     step_size = _DEFAULT_SOLVER_OPTIONS['step_size'],
     lbfgs_memory_level = _DEFAULT_SOLVER_OPTIONS['lbfgs_memory_level'],
-    max_iterations = _DEFAULT_SOLVER_OPTIONS['max_iterations'], verbose=True):
+    max_iterations = _DEFAULT_SOLVER_OPTIONS['max_iterations'],
+    class_weights = None, verbose=True):
     """
     Create a :class:`~graphlab.logistic_classifier.LogisticClassifier` (using
     logistic regression as a classifier) to predict the class of a binary
@@ -50,7 +51,11 @@ def create(dataset, target, features=None, l2_penalty=0.01, l1_penalty=0.0,
 
     target : string
         Name of the column containing the target variable. The values in this
-        column must be 0 or 1, of integer type.
+        column must of string or integer type. String target variables are
+        autumatically mapped to integers in alphabetical order of the variable
+        values. For example, a target variable with 'cat' and 'dog' as possible
+        values is mapped to 0 and 1 respectively with 0 being the base class 
+        and 1 being the reference class. 
 
     features : list[string], optional
         Names of the columns containing features. 'None' (the default) indicates
@@ -159,6 +164,12 @@ def create(dataset, target, features=None, l2_penalty=0.01, l1_penalty=0.0,
         iteration takes a considerable amount of time, reducing this parameter
         may speed up model training.
 
+    class_weights : {dict, `auto`}, optional
+
+        Weights the examples in the training data according to the given class
+        weights. If set to `None`, all classes are supposed to have weight one. The
+        `auto` mode set the class weight to be inversely proportional to number of
+        examples in the training data with the given class.
 
     verbose : bool, optional
         If True, print progress updates.
@@ -265,17 +276,15 @@ def create(dataset, target, features=None, l2_penalty=0.01, l1_penalty=0.0,
                         convergence_threshold = convergence_threshold, 
                         step_size = step_size, 
                         lbfgs_memory_level = lbfgs_memory_level, 
-                        max_iterations = max_iterations)
+                        max_iterations = max_iterations, 
+                        class_weights = class_weights)
 
     return LogisticClassifier(model.__proxy__)
 
 class LogisticClassifier(_Classifier):
     """
     Logistic regression models a binary target variable as a function of
-    several feature variables. An instance of this model can be created using
-    :func:`graphlab.logistic_classifier.create`.  Do not construct the model
-    directly. A detailed list of options are available in the create function.
-
+    several feature variables. 
 
     The :class:`~graphlab.logistic_classifier.logisticClassifier` uses
     a binary target variable :math:`y` instead of a scalar. For each
@@ -287,7 +296,6 @@ class LogisticClassifier(_Classifier):
     :class:`~graphlab.evaluation` toolkit gives you more flexible evaluation
     metrics.
 
-
     Given a set of features :math:`x_i`, and a label :math:`y_i \in \{0,1\}`,
     logistic regression interprets the probability that the label is in one class
     as a logistic function of a linear combination of the features.
@@ -308,6 +316,11 @@ class LogisticClassifier(_Classifier):
     where :math:`\lambda_1` is the ``l1_penalty`` and :math:`\lambda_2` is the
     ``l2_penalty``.
 
+    This model cannot be constructed directly.  Instead, use 
+    :func:`graphlab.logistic_classifier.create` to create an instance
+    of this model. A detailed list of parameter options and code samples 
+    are available in the documentation for the create function.
+
     Examples
     --------
     .. sourcecode:: python
@@ -566,7 +579,7 @@ class LogisticClassifier(_Classifier):
         the logistic function. Predicted classes are obtained by thresholding
         the predicted probabilities at 0.5. If you would like to threshold
         predictions at a different probability level, you can use the
-        Graphlab evaluation toolkit.
+        GraphLab evaluation toolkit.
 
         Parameters
         ----------
