@@ -11,6 +11,7 @@ from graphlab.toolkits._model import _get_default_options_wrapper
 from graphlab.toolkits._internal_utils import _raise_error_if_not_sframe
 from graphlab.toolkits._internal_utils import _map_unity_proxy_to_object
 from graphlab.toolkits._internal_utils import _toolkit_repr_print
+from graphlab.toolkits._internal_utils import _numeric_param_check_range
 from graphlab_util import cloudpickle as _cloudpickle
 
 import logging as _logging
@@ -43,10 +44,11 @@ class NeuralNetClassifier(_Classifier):
     machine learning, and has recently achieved great success in computer vision
     tasks such as object recognition.
 
-    Do NOT construct this model directly; use the
-    :func:`graphlab.neuralnet_classifier.create` function instead. The
-    documentation for the create() function also has additional details about
-    the model, as well as code samples.
+    This model cannot be constructed directly.  Instead, use
+    :func:`graphlab.neuralnet_classifier.create` to create an instance of
+    this model.
+    Additional details on parameter options and code samples are available in
+    the documentation for the create function.
 
     See Also
     --------
@@ -265,6 +267,106 @@ class NeuralNetClassifier(_Classifier):
         _mt._get_metric_tracker().track('toolkit.classifier.neuralnet_classifier.predict')
         return super(NeuralNetClassifier, self).predict(dataset, output_type=output_type)
 
+    def extract_features(self, dataset, layer_id=None):
+        """
+        Takes an input dataset, propagates each example through the network,
+        and returns an SArray of dense feature vectors, each of which is the concatenation
+        of all the hidden unit values at layer[layer_id]. These feature vectors
+        can be used as input to train another classifier such as a :py:class:`~graphlab.logistic_classifier.LogisticClassifier`,
+        an :py:class:`~graphlab.svm_classifier.SVMClassifier`, another
+        :py:class:`~graphlab.neuralnet_classifier.NeuralNetClassifier`, or a :py:class:`~graphlab.boosted_trees_classifier.BoostedTreesClassifier`.
+
+        We also are releasing a pre-trained model for ImageNet, as described by
+        Alex Krizhevsky et. al. It is located at
+        http://s3.amazonaws.com/GraphLab-Datasets/deeplearning/imagenet_model_iter45 .
+        Using it requires 256 x 256 x 3 images.
+        Please see Examples and References for more.
+
+
+        Parameters
+        ----------
+        dataset : SFrame
+            Dataset of new observations. Must include columns with the same
+            names as the features used for model training, but does not require
+            a target column. Additional columns are ignored.
+
+        layer_id : int , optional
+            The index of the layer in neuralnet at which the activations are
+            taken to be a dense feature vector. Must be a fully-connected layer.
+            Default is None, in which case the layer before the connection
+            layer to the output is used.
+
+
+        Returns
+        -------
+        out : SArray
+            An SArray of dtype array.array containing extracted features.
+
+        See Also
+        ------------
+        graphlab.deeplearning.layers
+
+        References
+        ----------
+        - Krizhevsky, Alex, Ilya Sutskever, and Geoffrey E. Hinton. "Imagenet
+        classification with deep convolutional neural networks." Advances in
+        neural information processing systems. 2012.
+
+        Examples
+        --------
+        >>> data = graphlab.SFrame('http://s3.amazonaws.com/GraphLab-Datasets/mnist/sframe/train6k')
+        >>> net = graphlab.deeplearning.get_builtin_neuralnet('mnist')
+        >>> m = graphlab.neuralnet_classifier.create(data,
+        ...                                          target='label',
+        ...                                          network=net,
+        ...                                          max_iterations=3)
+        >>> # Now, let's extract features from the last layer
+        >>> data['features'] = m.extract_features(data)
+        >>> # Now, let's build a new classifier on top of extracted features
+        >>> m = graphlab.classifier.create(data,
+        ...                                          features = ['features'],
+        ...                                          target='label')
+
+        Now, let's see how to load the ImageNet model, and use it for extracting
+        features after resizing the data:
+
+        >>> imagenet_model = graphlab.load_model('http://s3.amazonaws.com/GraphLab-Datasets/deeplearning/imagenet_model_iter45')
+        >>> data['image'] = graphlab.image_analysis.resize(data['image'], 256, 256, 3)
+        >>> data['imagenet_features'] = imagenet_model.extract_features(data)
+
+        """
+        _mt._get_metric_tracker().track('toolkit.classifier.neuralnet_classifier.extract_features')
+        _raise_error_if_not_sframe(dataset, "dataset")
+        options = dict()
+
+        net = self.get('network').layers
+        network_size = len(net) - 1
+        if layer_id is None:
+            if net[network_size]._type == "CONNECTION":
+                layer_id = network_size - 1
+            else:
+                layer_id = network_size - 2
+        _numeric_param_check_range("layer_id", layer_id, 0, network_size)
+
+        conv2flat = False
+        for i in range(0, layer_id + 1):
+            if net[i]._type == "CONNECTION" or net[i]._type == "TRANSITION":
+                conv2flat = True
+
+        if conv2flat is not True:
+            raise ValueError("Features must be extracted from either a network "
+                    "with non-image input or a layer after a FlattenLayer. "
+                    "Try extracting features from layer following a FlattenLayer.")
+
+        options.update({'model': self.__proxy__,
+                        'model_name': self.__name__,
+                        'dataset': dataset,
+                        'layer_id': layer_id})
+        target = _toolkits_main.run('neuralnet_feature_extraction', options)
+        return _map_unity_proxy_to_object(target['extracted'])
+
+
+
     def predict_topk(self, dataset, output_type="score", k=3):
         """
         Return top-k predictions for the ``dataset``, using the trained model.
@@ -594,7 +696,7 @@ def create(dataset, target,
         - ``bias_learning_rate`` : float, default unused
             Specify the learning rate for bias, overriding `learning_rate`.
 
-        - ``bias_momentum`` : float, default 0.0
+        - ``bias_momentum`` : float, default unused
             Specify the momentum for bias, overriding `momentum`.
 
         - ``bias_l2_regularization`` : float, default 0.0
@@ -620,7 +722,7 @@ def create(dataset, target,
         - ``learning_rate_step`` : int, default 1
             update the learning rate every learning_rate_step number of epochs.
 
-        - ``learrning_rate_gamma`` : float, default 0.1.
+        - ``learning_rate_gamma`` : float, default 0.1.
             learning decay param used in 'exponential_decay'
 
         - ``learning_rate_alpha`` : float, default 0.5
@@ -639,7 +741,7 @@ def create(dataset, target,
         - ``init_bias`` : float, default 0.0
             The initial value of the biases.
 
-        - ``divideby`` : float, default 0.0
+        - ``divideby`` : float, default 1.0
             The value by which to scale the input data before it is inserted into
             the network.
 
