@@ -28,7 +28,8 @@ def create(dataset, target, features=None, penalty=1.0, solver='auto',
     feature_rescaling=True,
     convergence_threshold = _DEFAULT_SOLVER_OPTIONS['convergence_threshold'],
     lbfgs_memory_level = _DEFAULT_SOLVER_OPTIONS['lbfgs_memory_level'],
-    max_iterations = _DEFAULT_SOLVER_OPTIONS['max_iterations'], verbose=True):
+    max_iterations = _DEFAULT_SOLVER_OPTIONS['max_iterations'], 
+    class_weights = None, verbose=True):
     """
     Create a :class:`~graphlab.svm_classifier.SVMClassifier` to predict the class of a binary
     target variable based on a model of which side of a hyperplane the example
@@ -46,7 +47,11 @@ def create(dataset, target, features=None, penalty=1.0, solver='auto',
 
     target : string
         Name of the column containing the target variable. The values in this
-        column must be 0 or 1, of integer type.
+        column must of string or integer type. String target variables are
+        autumatically mapped to integers in alphabetical order of the variable
+        values. For example, a target variable with 'cat' and 'dog' as possible
+        values is mapped to 0 and 1 respectively with 0 being the base class 
+        and 1 being the reference class. 
 
     features : list[string], optional
         Names of the columns containing features. 'None' (the default) indicates
@@ -127,6 +132,13 @@ def create(dataset, target, features=None, penalty=1.0, solver='auto',
         the model trained. Setting this to more than ``max_iterations`` has the
         same effect as setting it to ``max_iterations``.
 
+    class_weights : {dict, `auto`}, optional
+
+        Weights the examples in the training data according to the given class
+        weights. If set to `None`, all classes are supposed to have weight one. The
+        `auto` mode set the class weight to be inversely proportional to number of
+        examples in the training data with the given class.
+
     verbose : bool, optional
         If True, print progress updates.
 
@@ -197,7 +209,8 @@ def create(dataset, target, features=None, penalty=1.0, solver='auto',
                         feature_rescaling = feature_rescaling,
                         convergence_threshold = convergence_threshold,
                         lbfgs_memory_level = lbfgs_memory_level,
-                        max_iterations = max_iterations)
+                        max_iterations = max_iterations, 
+                        class_weights = class_weights)
 
     return SVMClassifier(model.__proxy__)
 
@@ -205,10 +218,7 @@ def create(dataset, target, features=None, penalty=1.0, solver='auto',
 class SVMClassifier(_Classifier):
     """
     Support Vector Machines can be used to predict binary target variable using
-    several feature variables.  An instance of this model can be created using
-    :func:`graphlab.svm_classifier.create`.  Do not construct the model directly.
-    Additional details about the model construction (along with code samples)
-    are documented with the :func:`graphlab.svm_classifier.create` function.
+    several feature variables.  
 
     The :py:class:`~graphlab.svm.SVMClassifier` model predicts a binary target
     variable given one or more feature variables. In an SVM model, the examples
@@ -231,6 +241,10 @@ class SVMClassifier(_Classifier):
 
     where :math:`\lambda` is the ``penalty`` parameter.
 
+    This model cannot be constructed directly.  Instead, use 
+    :func:`graphlab.svm_classifier.create` to create an instance of this model. 
+    Additional details on parameter options and code samples are available in 
+    the documentation for the create function.
 
     Examples
     --------
