@@ -400,7 +400,7 @@ class NeuralNet(_generic.CommonEqualityMixin):
         Override hyperparameters for this network
 
         >>> net.params['learning_rate'] = 0.1
-        >>> net.params['init_random'] = 'xaiver'
+        >>> net.params['init_random'] = 'xavier'
 
         See Also
         --------
