@@ -54,8 +54,10 @@ class KcoreModel(_ModelBase):
     | training_time | Total training time of the model                   |
     +---------------+----------------------------------------------------+
 
-    An instance of this model can be created using :func:`graphlab.kcore.create`.
-    Do NOT construct the model directly.
+    This model cannot be constructed directly.  Instead, use 
+    :func:`graphlab.kcore.create` to create an instance
+    of this model. A detailed list of parameter options and code samples 
+    are available in the documentation for the create function.
 
     See Also
     --------
