@@ -11,15 +11,15 @@ points, also stored in an SFrame.
 
 .. sourcecode:: python
 
-    >>> references = graphlab.SFrame({'label': [str(x) for x in range(3)],
-                                     'feature1': [0.98, 0.62, 0.11],
-                                     'feature2': [0.69, 0.58, 0.36]})    
-    >>> model = graphlab.nearest_neighbors.create(references, 'label')
+    >>> references = graphlab.SFrame({'feature1': [0.98, 0.62, 0.11],
+    ...                               'feature2': [0.69, 0.58, 0.36]})
+    >>> references = references.add_row_number(column_name='id')
+    >>> model = graphlab.nearest_neighbors.create(references, label='id')
 
-    >>> queries = graphlab.SFrame({'label': [str(x) for x in range(3)],
-                                  'feature1': [0.05, 0.61, 0.99],
-                                  'feature2': [0.06, 0.97, 0.86]})    
-    >>> model.query(queries, 'label', k=2)
+    >>> queries = graphlab.SFrame({'feature1': [0.05, 0.61, 0.99],
+    ...                            'feature2': [0.06, 0.97, 0.86]})
+    >>> queries = queries.add_row_number(column_name='id')    
+    >>> model.query(queries, label='id', k=2)
     +-------------+-----------------+----------------+------+
     | query_label | reference_label |    distance    | rank |
     +-------------+-----------------+----------------+------+
@@ -35,8 +35,10 @@ In addition to the API documentation, please see the data science `Gallery
 <http://www.graphlab.com/learn/gallery>`_, `How-tos
 <http://www.graphlab.com/learn/how-to>`_, and the `clustering chapter of the
 User Guide 
-<http://www.graphlab.com/learn/userguide.html#Modeling_data_Clustering_and_nearest_neighbors>`_
+<http://www.graphlab.com/learn/userguide/index.html#Modeling_data_Clustering_and_nearest_neighbors>`_
 for more details and extended examples.
 """
 
-from graphlab.toolkits.nearest_neighbors import nearest_neighbors
\ No newline at end of file
+from _nearest_neighbors import create
+from _nearest_neighbors import get_default_options
+from _nearest_neighbors import NearestNeighborsModel
