@@ -141,6 +141,12 @@ def _create(observation_data,
                 binary_target = True
 
     side_data = (user_data is not None) or (item_data is not None)
+    if user_data is not None:
+        if not isinstance(user_data, _SFrame):
+            raise TypeError('Provided user_data must be an SFrame.')
+    if item_data is not None:
+        if not isinstance(item_data, _SFrame):
+            raise TypeError('Provided item_data must be an SFrame.')
 
     if target is None:
         if ranking:
@@ -639,8 +645,8 @@ def precision_recall_by_user(observed_user_items,
     item_id = recommendations.column_names()[1]
 
     assert observed_user_items.num_rows() > 0, \
-           "Evaluating precision and recall requires a non-empty \
-            observed_user_items."
+           "Evaluating precision and recall requires a non-empty " + \
+           "observed_user_items."
     assert user_id in observed_user_items.column_names(), \
             "User column required in observed_user_items."
     assert item_id in observed_user_items.column_names(), \
@@ -659,7 +665,8 @@ def precision_recall_by_user(observed_user_items,
             'cutoffs': cutoffs}
 
     response = _graphlab.toolkits._main.run('evaluation_precision_recall_by_user', opts)
-    return _SFrame(None, _proxy=response['pr'])
+    sf = _SFrame(None, _proxy=response['pr'])
+    return sf.sort([user_id, 'cutoff'])
 
 
 def random_split_by_user(dataset,
@@ -709,8 +716,9 @@ def random_split_by_user(dataset,
 
     Examples
     --------
-    >>> sf = graphlab.SFrame('s3://GraphLab-Datasets/audioscrobbler')
-    >>> train, test = graphlab.recommender.random_split_by_user(sf, max_num_users=100)
+    >>> import graphlab as gl
+    >>> sf = gl.SFrame('s3://GraphLab-Datasets/audioscrobbler')
+    >>> train, test = gl.recommender.util.random_split_by_user(sf, max_num_users=100)
     """
 
     _mt._get_metric_tracker().track('toolkit.recsys.random_split_by_user')
@@ -1297,6 +1305,15 @@ class _Recommender(_Model):
             Contains the precision and recall at each cutoff value and each
             user in ``dataset``.
 
+        Examples
+        --------
+
+        >>> import graphlab as gl
+        >>> sf = gl.SFrame('s3://GraphLab-Datasets/audioscrobbler')
+        >>> train, test = gl.recommender.util.random_split_by_user(sf)
+        >>> m = gl.recommender.create(train)
+        >>> m.evaluate_precision_recall(test)
+
         See Also
         --------
         graphlab.recommender.util.precision_recall_by_user
@@ -1329,7 +1346,7 @@ class _Recommender(_Model):
              operations={'precision' : _Aggregate.MEAN('precision'),
                          'recall'    : _Aggregate.MEAN('recall')})
 
-
+        pr_agg = pr_agg[['cutoff', 'precision', 'recall']]
         ret["precision_recall_overall"] = pr_agg.sort("cutoff")
 
         return ret
@@ -1355,6 +1372,14 @@ class _Recommender(_Model):
             which are SFrames containing the average rmse for each user and
             item, respectively; and 'rmse_overall', which is a float.
 
+        Examples
+        --------
+        >>> import graphlab as gl
+        >>> sf = gl.SFrame('s3://GraphLab-Datasets/audioscrobbler')
+        >>> train, test = gl.recommender.util.random_split_by_user(sf)
+        >>> m = gl.recommender.create(train, target='target')
+        >>> m.evaluate_rmse(test, target='target')
+
         See Also
         --------
         graphlab.evaluation.rmse
@@ -1463,6 +1488,14 @@ class _Recommender(_Model):
             evaluation criteria) an :py:class:`~graphlab.SFrame` is returned
             with both of these metrics for each user at several cutoff values.
 
+        Examples
+        --------
+        >>> import graphlab as gl
+        >>> sf = gl.SFrame('s3://GraphLab-Datasets/audioscrobbler')
+        >>> train, test = gl.recommender.util.random_split_by_user(sf)
+        >>> m = gl.recommender.create(train, target='target')
+        >>> m.evaluate(test)
+
         See Also
         --------
         evaluate_precision_recall, evaluate_rmse, precision_recall_by_user
