@@ -397,36 +397,6 @@ def split_path_elements(url):
     else:
         return (url, "")
 
-def validate_feature_types(dataset, features, valid_feature_types):
-    if features is not None:
-        if not hasattr(features, '__iter__'):
-            raise TypeError("Input 'features' must be an iterable type.")
-
-        if not all([isinstance(x, str) for x in features]):
-            raise TypeError("Input 'features' must contain only strings.")
-
-    ## Extract the features and labels
-    if features is None:
-        features = dataset.column_names()
-
-    col_type_map = {
-        col_name: col_type for (col_name, col_type) in
-        zip(dataset.column_names(), dataset.column_types()) }
-
-    valid_features = []
-    for col_name in features:
-        if col_type_map.get(col_name) in valid_feature_types:
-            valid_features.append(col_name)
-
-    if len(valid_features) == 0:
-        if features is not None:
-            raise TypeError("No valid feature types specified")
-        else:
-            raise TypeError("The dataset does not contain any valid feature types")
-
-    return features
-
-
 def crossproduct(d):
     """
     Create an SFrame containing the crossproduct of all provided options.
@@ -468,3 +438,74 @@ def crossproduct(d):
     return SArray(sa).unpack(column_name_prefix='')
 
 
+def get_graphlab_object_type(url):
+    '''
+    Given url where a GraphLab object is persisted, return the GraphLab object
+    type: 'model', 'graph', 'sframe', or 'sarray'
+    '''
+    ret = _glconnect.get_unity().get_graphlab_object_type(make_internal_url(url))
+
+    # to be consistent, we use sgraph instead of graph here
+    if ret == 'graph':
+        ret = 'sgraph'
+    return ret
+
+class _distances_utility(object):
+    """
+    A collection of distance functions.
+
+    Examples
+    --------
+
+    >>> import graphlab as gl
+    >>> a = [1, 2, 3]
+    >>> b = [4, 5, 6]
+    >>> gl.distances.euclidean(a, b)
+
+    """
+    def __init__(self):
+        distances = \
+          {'euclidean':         lambda x, y: graphlab.extensions._distances.euclidean(x, y),
+           'squared_euclidean': lambda x, y: graphlab.extensions._distances.squared_euclidean(x, y),
+           'manhattan':         lambda x, y: graphlab.extensions._distances.manhattan(x, y),
+           'cosine':            lambda x, y: graphlab.extensions._distances.cosine(x, y),
+           'dot_product':       lambda x, y: graphlab.extensions._distances.dot_product(x, y),
+           'levenshtein':       lambda x, y: graphlab.extensions._distances.levenshtein(x, y),
+           'jaccard':           lambda x, y: graphlab.extensions._distances.jaccard(x, y),
+           'weighted_jaccard':  lambda x, y: graphlab.extensions._distances.weighted_jaccard(x, y)
+          }
+
+        # TODO: It would be better to grab these from the __doc__ string that
+        # is associated with the extension, but that would require an engine start.
+        docstrings = \
+          {'euclidean':         "Compute the Euclidean distance between two dictionaries or two lists of equal length.",
+           'squared_euclidean': "Compute the squared Euclidean distance between two dictionaries or two lists of equal length.",
+           'manhattan':         "Compute the Manhattan distance between between two dictionaries or two lists of equal length.",
+           'cosine':            "Compute the cosine distance between between two dictionaries or two lists of equal length.",
+           'dot_product':       "Compute the dot_product between two dictionaries or two lists of equal length.",
+           'levenshtein':       "Compute the Levenshtein distance between between strings.",
+           'jaccard':           "Compute the Jaccard distance between between two dictionaries.",
+           'weighted_jaccard':  "Compute the weighted Jaccard distance between between two dictionaries."
+           }
+
+        # Copy lambda along with its corresponding docstring
+        self.__dict__ = {}
+        for k in docstrings:
+            d = distances[k]
+            d.__doc__ = docstrings[k]
+            self.__dict__[k] = d
+
+
+def _get_distance(distance_name):
+    """
+    Get the actual function for a distance from the name of the distance.
+    """
+    def ismatch(distance, actual):
+        return (distance == actual) or (distance == '_distances.'+actual)
+
+    for distance, fn in graphlab.distances.__dict__.iteritems():
+        if ismatch(distance_name, distance):
+            return fn
+
+    raise ValueError('Distance not recognized')
+
