@@ -1,21 +1,39 @@
 import os
 import time
-
+import logging
 
 class GraphLabConfig:
 
     __slots__ = ['graphlab_server', 'server_addr', 'server_bin', 'log_dir', 'unity_metric',
-                 'mode', 'version', 'librato_user', 'librato_token', 'mixpanel_user']
+                 'mode', 'version', 'librato_user', 'librato_token', 'mixpanel_user',
+                 'log_rotation_interval','log_rotation_truncate']
 
     def __init__(self, server_addr=None):
         if not server_addr:
             server_addr = 'default'
         self.server_addr = server_addr
         gl_root = os.path.abspath(os.path.join(os.path.dirname(os.path.realpath(__file__)), '..', 'graphlab'))
+        self.log_rotation_interval = 86400
+        self.log_rotation_truncate = 8
         if "GRAPHLAB_UNITY" in os.environ:
             self.server_bin = os.environ["GRAPHLAB_UNITY"]
         elif os.path.exists(os.path.join(gl_root, "unity_server")):
             self.server_bin = os.path.join(gl_root, "unity_server")
+
+        if "GRAPHLAB_LOG_ROTATION_INTERVAL" in os.environ:
+            tmp = os.environ["GRAPHLAB_LOG_ROTATION_INTERVAL"]
+            try:
+                self.log_rotation_interval = int(tmp)
+            except:
+                logging.getLogger(__name__).warning("GRAPHLAB_LOG_ROTATION_INTERVAL must be an integral value")
+
+        if "GRAPHLAB_LOG_ROTATION_TRUNCATE" in os.environ:
+            tmp = os.environ["GRAPHLAB_LOG_ROTATION_TRUNCATE"]
+            try:
+                self.log_rotation_truncate = int(tmp)
+            except:
+                logging.getLogger(__name__).warning("GRAPHLAB_LOG_ROTATION_TRUNCATE must be an integral value")
+
         if "GRAPHLAB_LOG_PATH" in os.environ:
             log_dir = os.environ["GRAPHLAB_LOG_PATH"]
         else:
@@ -47,7 +65,7 @@ class GraphLabConfig:
           self.mixpanel_user = graphlab_env.mixpanel_user
           self.graphlab_server = graphlab_env.graphlab_server
 
-          # NOTE: Rmemeber to update slots if you are adding any config parameters to this file.
+          # NOTE: Remember to update slots if you are adding any config parameters to this file.
 
     def get_unity_log(self):
         ts = str(int(time.time()))
